/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.oxapi.streaming;

import java.io.IOException;
import java.io.InputStream;
import org.apache.commons.fileupload.FileItemIterator;
import org.apache.commons.fileupload.FileItemStream;
import org.apache.commons.fileupload.FileUploadException;

/**
 * Wraps an org.apache.commons.fileupload.FileItem into an input stream
 */
public class FileItemInputStream extends InputStream {

    private final FileItemIterator fileItemIterator;
    private final String fieldName;
    private String itemName;
    private String contentType;
    private InputStream fileStream;

    /**
     * Constructor
     * 
     * @param fieldName The name of the file item
     * @param fileItemIterator The iterator to a collection of file items
     */
    public FileItemInputStream(String fieldName, FileItemIterator fileItemIterator) {
        this.fieldName = fieldName;
        this.fileItemIterator = fileItemIterator;
    }

    /**
     * Searches for the file item and opens a stream to it
     * 
     * @throws IOException
     * @throws FileUploadException
     */
    private void EnsureStreamIsLoaded() throws IOException, FileUploadException {
        if (fileStream == null) {
            while (fileItemIterator.hasNext()) {
                FileItemStream item = fileItemIterator.next();
                if (item.getFieldName().equals(this.fieldName)) {
                    fileStream = item.openStream();
                    this.itemName = item.getName();
                    this.contentType = item.getContentType();
                    break;
                }
            }
        }
    }

    /**
     * @return The item's name
     * @throws Exception
     */
    public String getItemName() throws Exception {
        EnsureStreamIsLoaded();
        return this.itemName;
    }

    /**
     * @return The item's content type
     * @throws Exception
     */
    public String getContentType() throws Exception {
        EnsureStreamIsLoaded();
        return this.contentType;
    }

    @Override
    public int read() throws IOException {
        try {
            EnsureStreamIsLoaded();
            return fileStream.read();
        } catch (FileUploadException e) {
            throw new IOException(e.getMessage());
        }
    }

    @Override
    public int read(byte[] b) throws IOException {
        try {
            EnsureStreamIsLoaded();
            return fileStream.read(b);
        } catch (FileUploadException e) {
            throw new IOException(e.getMessage());
        }
    }

    @Override
    public int read(byte[] b, int off, int len) throws IOException {
        try {
            EnsureStreamIsLoaded();
            return fileStream.read(b, off, len);
        } catch (FileUploadException e) {
            throw new IOException(e.getMessage());
        }
    }

}
